/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：信息安全性设计准则检查插件
 * 项目描述：用于检查源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.java.rules.checkers;

import org.sonar.check.Rule;
import org.sonar.java.ast.parser.ArgumentListTreeImpl;
import org.sonar.java.model.InternalSyntaxToken;
import org.sonar.java.model.expression.LiteralTreeImpl;
import org.sonar.plugins.java.api.IssuableSubscriptionVisitor;
import org.sonar.plugins.java.api.tree.*;

import java.util.Collections;
import java.util.List;

/**
 * 使用RSA最优加密填充
 *
 * @author RenFengJiang
 * @date 2024/1/11
 */
@Rule(key = "RSAEncryptionChecker")
public class RSAEncryptionChecker extends IssuableSubscriptionVisitor {

    @Override
    public List<Tree.Kind> nodesToVisit() {
        /**
         * Tree.Kind.METHOD：方法节点
         * Tree.Kind.BLOCK：方法的代码块节点
         * Tree.Kind.METHOD_INVOCATION： 方法的调用节点
         */
        return Collections.singletonList(Tree.Kind.METHOD_INVOCATION);
    }

    @Override
    public void visitNode(Tree tree) {
        MethodInvocationTree methodInvocationTree = (MethodInvocationTree) tree;
        ExpressionTree expressionTree = methodInvocationTree.methodSelect();
//        获取到方法调用
        if(expressionTree instanceof MemberSelectExpressionTree){
            MemberSelectExpressionTree memberSelectExpressionTree = (MemberSelectExpressionTree) expressionTree;
//            判断是否符合标准
            if("Cipher".equals(memberSelectExpressionTree.expression().toString()) && "getInstance".equals(memberSelectExpressionTree.identifier().name())){
//                获取参数列表
                Arguments arguments = methodInvocationTree.arguments();
                if(arguments instanceof ArgumentListTreeImpl){
//                    获取到参数列表
                    ArgumentListTreeImpl argumentListTree = (ArgumentListTreeImpl) arguments;
                    for (ExpressionTree argument : argumentListTree) {
                        if(argument instanceof LiteralTreeImpl){
                            LiteralTreeImpl literalTree = (LiteralTreeImpl) argument;
//                            获取到其中的参数
                            SyntaxToken syntaxToken = literalTree.token();
                            if(syntaxToken instanceof InternalSyntaxToken){
                                InternalSyntaxToken internalSyntaxToken = (InternalSyntaxToken) syntaxToken;
//                                对参数进行判断判断是否符合要求
                                if(!(internalSyntaxToken.text().startsWith("\"RSA") && internalSyntaxToken.text().contains("OAEPWithSHA-256AndMGF1Padding"))){
                                    context.reportIssue(this,tree,"使用RSA最优加密填充");
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}
