/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：信息安全性设计准则检查插件
 * 项目描述：用于检查源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */

package com.keyware.sonar.java.utils;

import org.sonar.api.batch.fs.InputFile;
import org.sonar.api.batch.fs.internal.TestInputFileBuilder;
import org.sonar.plugins.html.analyzers.ComplexityVisitor;
import org.sonar.plugins.html.analyzers.PageCountLines;
import org.sonar.plugins.html.api.HtmlConstants;
import org.sonar.plugins.html.lex.PageLexer;
import org.sonar.plugins.html.lex.VueLexer;
import org.sonar.plugins.html.visitor.DefaultNodeVisitor;
import org.sonar.plugins.html.visitor.HtmlAstScanner;
import org.sonar.plugins.html.visitor.HtmlSourceCode;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.nio.charset.StandardCharsets;
import java.util.List;

/**
 * Html测试工具类
 *
 * @author GuoXin
 * @date 2024/1/20
 */
public class HtmlTestHelper {

    private HtmlTestHelper() {
    }

    public static HtmlSourceCode scan(File file, DefaultNodeVisitor visitor) {
        FileReader fileReader;
        try {
            fileReader = new FileReader(file);
        } catch (FileNotFoundException e) {
            throw new IllegalStateException(e);
        }

        HtmlSourceCode result = new HtmlSourceCode(
                new TestInputFileBuilder("key", file.getPath())
                        .setLanguage(HtmlConstants.LANGUAGE_KEY)
                        .setType(InputFile.Type.MAIN)
                        .setModuleBaseDir(new File(".").toPath())
                        .setCharset(StandardCharsets.UTF_8)
                        .build()
        );

        HtmlAstScanner walker = new HtmlAstScanner(List.of(new PageCountLines(), new ComplexityVisitor()));
        PageLexer lexer = file.getName().endsWith(".vue") ? new VueLexer() : new PageLexer();
        walker.addVisitor(visitor);
        walker.scan(
                lexer.parse(fileReader),
                result
        );

        return result;
    }

}
